// Mixture constants
const decl MIXTURE_WEIGHT1 = 0.37038;
const decl MIXTURE_MU1 = 0.81306;
const decl MIXTURE_SIGMA1 = 1.0657;
const decl MIXTURE_WEIGHT2 = 0.62962;
const decl MIXTURE_MU2 = -0.47828;
const decl MIXTURE_SIGMA2 = 0.54998;

/*
	Base functions for the normal distribution.
*/

dlogn(const x) {
	return -x;
}

/*
	Base functions for the standardized t distribution.
*/

rannormt(const r, const c, const df) {
	return rant(r, c, df) / sqrt(df / (df - 2));
}

densnormt(const ma, const df) {
	return denst(ma * sqrt(df / (df - 2)), df) * sqrt(df / (df - 2));
}

dlognormt(const x, const df) {
	return -(df+1)*x / (df + x^2 - 2);
}

/*
	Base functions for the chi-square-like mixture.
*/

ranchimix(const r, const c) {
	decl choice = ranu(r, c) .> MIXTURE_WEIGHT2;
	return choice .* (MIXTURE_MU1 + MIXTURE_SIGMA1 * rann(r, c)) + (1 - choice) .* (MIXTURE_MU2 + MIXTURE_SIGMA2 * rann(r, c));
}

denschimix(const ma) {
	return (MIXTURE_WEIGHT1 * densn((ma - MIXTURE_MU1) / MIXTURE_SIGMA1) / MIXTURE_SIGMA1)
		+
	(MIXTURE_WEIGHT2 * densn((ma - MIXTURE_MU2) / MIXTURE_SIGMA2) / MIXTURE_SIGMA2);
}

dlogchimix(const x) {
	decl ddenschimix = (MIXTURE_WEIGHT1 * -(x - MIXTURE_MU1) * densn((x - MIXTURE_MU1) / MIXTURE_SIGMA1) / MIXTURE_SIGMA1^3)
			+
		(MIXTURE_WEIGHT2 * -(x - MIXTURE_MU2) * densn((x - MIXTURE_MU2) / MIXTURE_SIGMA2) / MIXTURE_SIGMA2^3);
	return ddenschimix / denschimix(x);
}