/*
	Assign each element of a parameter vector to variable addresses

	Example:
		decl mu, sigma;
		GetParameters(par, &mu, &sigma);
		// equivalent to mu = par[0]; sigma = par[1];
*/
GetParameters(const par, ...) {
	decl args = va_arglist();
	for(decl i = 0; i < rows(par); i++) {
		args[i][0] = par[i];
	}
}

/*
	Transform restricted parameters into unrestricted and back.

	par: the parameter vector to be transformed
	opt: a vector with the same length as par
		opt[i] = 0: par[i] remains unaffected
		opt[i] = 1: par[i] is transformed from R+ to R (useful for optimization of positive parameters)
		opt[i] = -1: par[i] is transformed from R to R+ (to retrieve original parameters after optimization)
		opt[i] = 2: par[i] is transformed from (0, 1) to R (for optimization of parameters in (0,1))
		opt[i] = -2: par[i] is transformed from R to (0, 1)
*/
Transform(const par, const opt) {
	decl newpar = par;
	newpar[vecindex(opt .== 1)] = log(newpar[vecindex(opt .== 1)]);
	newpar[vecindex(opt .== -1)] = exp(newpar[vecindex(opt .== -1)]);
	newpar[vecindex(opt .== 2)] = -log(-1 + (1/newpar[vecindex(opt .== 2)]));
	newpar[vecindex(opt .== -2)] = 1 / (1 + exp(-newpar[vecindex(opt .== -2)]));
	return newpar;
}