#include <oxstd.oxh>
#include <oxdraw.oxh>
#include <oxprob.oxh>
#include <oxfloat.oxh>
#import <maximize>

#include "Misc.ox"
#include "Base.ox"
#include "StandardizedKernelEstimator2.ox"
#include "Components.ox"

/*
	SEE PROCESS.PY
*/

main () {
	decl T = 2000;
	decl replications = 1000;
	decl K = 1000; // how much data to use for estimation

	// two columns: RMSE in, RMSE out
	decl MLE_results = <>;
	decl QMLE_results = <>;
	decl t_results = <>;
	decl SP_results = <>;

	decl succes = 0;
	
	while(succes < replications) {
		decl truevol = Vol_%VOL%(T);
		decl data = %RAN% .* truevol;
		decl initpar = <0, 2, 0.3, 0.9>;
		decl inittpar = initpar ~ <5>;

		decl par1 = initpar, code1,	ft1, epst1,
			par2 = initpar, code2, ft2, epst2,
			par3 = inittpar, code3,	ft3, epst3,
			par4 = initpar, code4, ft4, epst4;

		code1 = %MLE%(data, K, &par1, &ft1, &epst1);		
		code2 = GAS_QMLE(data, K, &par2, &ft2, &epst2);
		code3 = GAS_t(data, K, &par3, &ft3, &epst3);
		code4 = GAS_SP1(data, K, &par4, &ft4, &epst4);

		if(code1+code2+code3+code4 == 0) {
			MLE_results |= sqrt(meanr(sqr(exp(ft1[:K-1]/2) - truevol[:K-1]))) ~ sqrt(meanr(sqr(exp(ft1[K:]/2) - truevol[K:])));
			QMLE_results |= sqrt(meanr(sqr(exp(ft2[:K-1]/2) - truevol[:K-1]))) ~ sqrt(meanr(sqr(exp(ft2[K:]/2) - truevol[K:])));
			t_results |= sqrt(meanr(sqr(exp(ft3[:K-1]/2) - truevol[:K-1]))) ~ sqrt(meanr(sqr(exp(ft3[K:]/2) - truevol[K:])));
			SP_results |= sqrt(meanr(sqr(exp(ft4[:K-1]/2) - truevol[:K-1]))) ~ sqrt(meanr(sqr(exp(ft4[K:]/2) - truevol[K:])));
			succes++;
		}
		
	}

	// combine results
	decl finalresults = zeros(4, 2);
	
	finalresults[0][] = quantilec(MLE_results, 0.5);
	finalresults[1][] = quantilec(QMLE_results, 0.5);
	finalresults[2][] = quantilec(t_results, 0.5);
	finalresults[3][] = quantilec(SP_results, 0.5);

	finalresults[][1] /= finalresults[1][0]; // normalize this one first
	finalresults[][0] /= finalresults[1][0];

	savemat("Table2_%NAME%_%VOL%.xlsx", finalresults);
}